package at.ac.uibk.gitsearch.repository.search;


import at.ac.uibk.gitsearch.config.ApplicationProperties;
import at.ac.uibk.gitsearch.es.model.DocumentInfo;
import at.ac.uibk.gitsearch.service.dto.DocumentInfoDTO;
import at.ac.uibk.gitsearch.service.dto.SearchResultDto;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import org.apache.lucene.search.vectorhighlight.BreakIteratorBoundaryScanner;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.stereotype.Repository;

import java.io.IOException;
import java.text.BreakIterator;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

@Repository
public class HighlightRepositoryImpl implements HighlightRepository {


    private final RestHighLevelClient elasticsearchClient;

    private final ApplicationProperties properties;

    public HighlightRepositoryImpl(RestHighLevelClient elasticsearchClient, ApplicationProperties properties) {
        this.elasticsearchClient = elasticsearchClient;
        this.properties = properties;
    }

    @Override
    public SearchResultDto customSearchWithHighlightedSummary(String query, int pageSize, int page, List<String> selectedRepository, List<String> selectedUniversity, List<String> selectedFileFormat) throws IOException {
        SearchRequest searchRequest = new SearchRequest(HighlightRepositoryConstants.INDEX);

        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
            .must(QueryBuilders.queryStringQuery(query).defaultField(HighlightRepositoryConstants.CONTENT_FIELD));

        if (selectedRepository.size() > 0) {
            queryBuilder.must(QueryBuilders.termsQuery(HighlightRepositoryConstants.REPOSITORY_FIELD, selectedRepository));
        }
        if (selectedUniversity.size() > 0) {
            queryBuilder.must(QueryBuilders.termsQuery(HighlightRepositoryConstants.SUB_GROUP_FIELD, selectedUniversity));
        }
        if (selectedFileFormat.size() > 0) {
            queryBuilder.must(QueryBuilders.termsQuery(HighlightRepositoryConstants.FILE_EXTENSION_FIELD, selectedFileFormat));
        }

        char[] boundaryChars = {'\n'};

        HighlightBuilder highlightBuilder = new HighlightBuilder()
            .highlighterType("plain")
            .preTags(properties.getSearch().getHighlightPre())
            .postTags(properties.getSearch().getHighlightPost())
            .field(HighlightRepositoryConstants.CONTENT_FIELD)
            .boundaryChars(boundaryChars)
            .boundaryScannerType(HighlightBuilder.BoundaryScannerType.SENTENCE)
            .boundaryMaxScan(1000)
            .fragmentSize(200);

        String[] includes = {};
        String[] excludes = {HighlightRepositoryConstants.CONTENT_FIELD};

        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        sourceBuilder.query(queryBuilder)
            .highlighter(highlightBuilder)
            .size(pageSize)
            .from((page - 1) * pageSize)
            .fetchSource(includes, excludes);

        searchRequest.source(sourceBuilder);

        SearchResponse searchResponse = elasticsearchClient.search(searchRequest, RequestOptions.DEFAULT);

        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.registerModule(new JavaTimeModule());
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

        List<DocumentInfoDTO> documentInfoDTOS = new ArrayList<>();
        for (SearchHit searchHit : searchResponse.getHits().getHits()) {
            DocumentInfo documentInfo = objectMapper.readValue(searchHit.getSourceAsString(), DocumentInfo.class);
            Map<String, HighlightField> highlightFields = searchHit.getHighlightFields();
            documentInfo.setContent(highlightFields.get(HighlightRepositoryConstants.CONTENT_FIELD).getFragments()[0].string());//.replaceAll("(\r\n|\n\r|\r|\n)", "<br />").replaceAll("\t", "&nbsp;&nbsp;&nbsp;&nbsp;"));
            documentInfoDTOS.add(new DocumentInfoDTO(documentInfo));
        }
        long hitCount = searchResponse.getHits().getTotalHits();
        return new SearchResultDto(documentInfoDTOS, null, hitCount);
    }
}
