import {Component, OnInit, OnDestroy} from '@angular/core';
import {Subject, Subscription} from 'rxjs';

import {LoginModalService} from 'app/core/login/login-modal.service';
import {AccountService} from 'app/core/auth/account.service';
import {Account} from 'app/core/user/account.model';
import {HttpResponse} from "@angular/common/http";
import {SearchResultService} from "app/home/search/search-result.service";
import {ActivatedRoute, Router} from "@angular/router";
import {ISearchResult} from "app/shared/model/search-result.model";
import {IDocumentInfo} from "app/shared/model/document-info.model";
import {QueryParam, QueryParamBuilder, QueryParamGroup} from "@ngqp/core";
import {takeUntil} from "rxjs/operators";
import {IFrequency} from "app/shared/model/frequency.model";
import * as Prism from "prismjs";
import 'prismjs/components/prism-scss';
import 'prismjs/components/prism-c';
import 'prismjs/components/prism-cpp';
import 'prismjs/components/prism-java';
import 'prismjs/components/prism-python';
import 'prismjs/components/prism-markdown';
import {LanguageDefinition} from "prismjs";

@Component({
  selector: 'jhi-home',
  templateUrl: './home.component.html',
  styleUrls: ['home.scss'],
  // encapsulation: ViewEncapsulation.ShadowDom,
})
export class HomeComponent implements OnInit, OnDestroy {
  account: Account | null = null;
  authSubscription?: Subscription;
  searchResultTotal?: ISearchResult;
  searchResultPage?: ISearchResult;
  currentSearch: string | null;
  currentPage = 1;

  page = 1;
  pageSize = 4;

  public selectedRepository: IFrequency<string>[];
  public selectedUniversity: IFrequency<string>[];
  public selectedFileFormat: IFrequency<string>[];

  public paramGroup: QueryParamGroup;
  private componentDestroyed$ = new Subject<void>();

  constructor(protected searchResultService: SearchResultService, private accountService: AccountService, private loginModalService: LoginModalService, protected activatedRoute: ActivatedRoute, private router: Router, private qpb: QueryParamBuilder) {
    this.currentSearch = null;
    this.selectedRepository = [];
    this.selectedUniversity = [];
    this.selectedFileFormat = [];

    this.paramGroup = qpb.group({
      searchText: qpb.stringParam('q', {
        debounceTime: 200,
      }),
      page: qpb.numberParam('page', {
        emptyOn: 1,
      }),
    });

    this.paramGroup.valueChanges.pipe(
      takeUntil(this.componentDestroyed$)
    ).subscribe(value => {
      this.page = value.page;
      this.search(value.searchText, value.page);
    });
  }

  private static highlight(content: string, language: LanguageDefinition) : string {
    const elements = content.split(/(<mark><strong>|<\/strong><\/mark>)/g);
    for (let i = 0; i < elements.length; ++i) {
      if (elements[i] === '<mark><strong>' || elements[i] === '</strong></mark>') {
        continue;
      }
      elements[i] =  Prism.highlight(elements[i], language, language);
    }
    return elements.join('');
  }

  public highlightCode(doc: IDocumentInfo): string {
    Prism.plugins.KeepMarkup = true;
    switch (doc.fileFormat) {
      case "C":
        return HomeComponent.highlight(doc.content, Prism.languages.c);
      case "C++":
        return HomeComponent.highlight(doc.content, Prism.languages.cpp);
      case "Java":
        return HomeComponent.highlight(doc.content, Prism.languages.java);
      case "Python":
        return HomeComponent.highlight(doc.content, Prism.languages.python);
      case "Markdown":
        return HomeComponent.highlight(doc.content, Prism.languages.markdown);
      default:
        return doc.content;
    }

  }

  filter(): void {
    if (!this.searchResultTotal?.hitCount || this.searchResultTotal.hitCount <= 0) {
      return;
    }
    this.loadPage();
  }

  search(query: string, page: number): void {
    if (this.currentSearch === query && this.currentPage === page) {
      return;
    }
    this.currentSearch = query;
    this.currentPage = page;
    this.loadTotal();
    this.loadPage();
  }

  ngOnInit(): void {
    this.loadTotal();
    this.loadPage();
    this.authSubscription = this.accountService.getAuthenticationState().subscribe(account => (this.account = account));
  }

  loadTotal(): void {
    if (this.currentSearch) {
      this.searchResultService.searchTotals({
        query: this.currentSearch
      })
        .subscribe(
          (res: HttpResponse<ISearchResult>) => {
            this.searchResultTotal = res.body || undefined;
          }
        );
    }
  }

  loadPage(): void {
    if (this.currentSearch) {
      this.searchResultService.search({
        query: this.currentSearch,
        pageSize: this.pageSize,
        page: this.page,
        selectedRepository: this.selectedRepository.map(value => value.key),
        selectedUniversity: this.selectedUniversity.map(value => value.key),
        selectedFileFormat: this.selectedFileFormat.map(value => value.key)
      })
        .subscribe(
          (res: HttpResponse<ISearchResult>) => {
            this.searchResultPage = res.body || undefined;
          }
        );
    }
  }

  isAuthenticated(): boolean {
    return this.accountService.isAuthenticated();
  }

  login(): void {
    this.loginModalService.open();
  }

  ngOnDestroy(): void {
    if (this.authSubscription) {
      this.authSubscription.unsubscribe();
    }
    this.componentDestroyed$.next();
    this.componentDestroyed$.complete();
  }

  onClickMe(documentInfo: IDocumentInfo): void {
    window.location.href = documentInfo.path;
  }

  public get repos(): IFrequency<string>[] {
    return this.searchResultTotal?.metaDataDto.repositories || [];
  }

  public get documentInfos(): IDocumentInfo[] {
    return this.searchResultPage?.documentInfos || [];
  }

  public get university(): IFrequency<string>[] {
    return this.searchResultTotal?.metaDataDto.university || [];
  }

  public get fileFormat(): IFrequency<string>[] {
    return this.searchResultTotal?.metaDataDto.fileFormat || [];
  }

  public get pageParam(): QueryParam<number> {
    return this.paramGroup.get('page') as QueryParam<number>;
  }

  public onPageChange(page: number): void {
    this.pageParam.setValue(page);
  }

  public get hitCount(): number {
    const hits = this.searchResultPage?.hitCount;
    return hits ? hits : 0;
  }


}
